//DS18B20温度测量 并通过计算机串口送出
module DS18B20
	(
		Clk,
		Rst,
     	DQ,   //18B20的数据端口
		Txd,
		Led,
		FinishFlag,
		Data16b
		//--
		//TxClk
	
	);
	
input Rst,Clk;
output Txd;
output reg [2:0] Led;//指示灯
inout DQ;
output reg [15:0] Data16b;//输出的16位温度值
output reg FinishFlag;  //输出完成指示信号


reg DQ_Reg;//DQ_Reg的map reg
reg En;//使能
wire Clk1M,TxClk;//;

reg [7:0] TXData;//发送的数据
reg XmitPlus;
wire TxFinish;
reg [13:0] Cnt;
reg [7:0] BitCnt;
reg [7:0] ByteCnt;
reg [7:0] DataTmp;
reg [9:0] DelayCnt1;
reg [9:0] DelayCnt2;
reg [7:0] ReadData;//从器件读出来的数据
reg [7:0] TL;//测量的温度低字节
reg [7:0] TH;//测量的温度高字节

//48分频的,产生1M的时钟
defparam Gen_Clk1M.divdFACTOR=24,Gen_Clk1M.divdWIDTH=4;//分频时钟
gen_divd Gen_Clk1M(.reset(!Rst),.clkin(Clk),.clkout(Clk1M));//端口名称关联


//416分频的,产生115200的时钟
defparam Gen_TxClk.divdFACTOR=208,Gen_TxClk.divdWIDTH=7;//分频时钟
gen_divd Gen_TxClk(.reset(!Rst),.clkin(Clk),.clkout(TxClk));//端口名称关联

//串口发送
uart_x UartTx(.Bclk(TxClk),.Resett(!Rst),.Txd(Txd),
				.XmitCmdp(XmitPlus),.TxdBuf(TXData),.TxDone(TxFinish));

assign DQ=En?DQ_Reg:1'bz;	


always @ (posedge Clk1M or negedge Rst)	
begin
	if(!Rst)
		begin
			Led<=3'b000;//全灭
			En<=1'b1;//输出
			DQ_Reg<=1'b1;
			Cnt<=0;
			XmitPlus<=1'b0;
			DelayCnt1<=0;
			DelayCnt2<=0;
			FinishFlag<=1'b0;
		end
	else
		begin
		Cnt<=Cnt+1'b1;
		case (Cnt)
		0:begin En<=1'b1;DQ_Reg<=1'b0;Led<=3'b000;end//输出 保持总线低电平500us(至少480us)
			500:begin En<=1'b0;end//释放总线120us,自动拉高(等待15~60us之间,从端拉低维持60~240us之间)
			621:begin   
					if (DQ==1'b0) //为低，初始完毕 跳出
						begin
							Cnt<=622;
							//Led[0]<=1'b1;
						end
					else		//继续等待 检测
						Cnt<=621;  
				end
				
			// 这个时间要足够长，延时等待让这个复位应答阶段结	
			650:begin
					DelayCnt1<=DelayCnt1+1;
					if (DelayCnt1==100)
						begin
							DelayCnt2<=DelayCnt2+1;
							if (DelayCnt2==100)
								begin
									Cnt<=680;//跳到下一个阶段  第三次复位
									DelayCnt1<=0;
									DelayCnt2<=0;
								end
							else
								begin
									Cnt<=650;//继续延时
									DelayCnt1<=0;
								end
						end
					else
						begin
							Cnt<=650;//继续延时
						end
				 end	
		
			1005:begin  DataTmp<=8'HCC;BitCnt<=0;end //忽略ROM匹配，复位后接着必是写该ROM命令		
			1011:begin En<=1'b1;DQ_Reg<=1'b0;end//保持总线低电平10us
			1021:begin DQ_Reg<=DataTmp[BitCnt];end//LSB first  每个数据维持80us左右,让18b20采样(采样时间是第15～第45us)
			1110:begin DQ_Reg<=1'b1;end //拉高
			1111:begin BitCnt<=BitCnt+1'b1;end
			1115:begin		
					if (BitCnt==8)  //一个字节发送完毕
						begin
							BitCnt<=0;
							ByteCnt<=ByteCnt+1'b1;
							if (ByteCnt==0)
								begin
									DataTmp<=8'H4E;//设置暂存reg
									Cnt<=1011;//继续发送下一个字节
								end
							else if (ByteCnt==1)
								begin
									DataTmp<=8'H64;//温度上限
									Cnt<=1011;//继续发送下一个字节
								end
							else if (ByteCnt==2)
								begin
									DataTmp<=8'H8A;//温度下限
									Cnt<=1011;//继续发送下一个字节
								end
							else if (ByteCnt==3)	
								begin
									DataTmp<=8'H7F;//配置reg  1F--9位精度  7F-12位精度
									Cnt<=1011;//继续发送下一个字节
								end
							else if (ByteCnt==4)	
								begin
									Cnt<=1130;//跳到第二次复位里
									ByteCnt<=0;
								end		
						end
					else
						Cnt<=1011;//继续发送该字节的bit位
				 end
				
			//第二次复位
			1130:begin En<=1'b1;DQ_Reg<=1'b0;end//输出 保持总线低电平500us(至少480us)
			1630:begin En<=1'b0;end//释放总线120us,自动拉高(等待15~60us之间,从端拉低维持60~240us之间)
			1731:begin
					if (DQ==1'b0) //为低，初始完毕 跳出
						begin
							Cnt<=1732;
							//Led[1]<=1'b1;
						end
					else		//继续等待 检测
						Cnt<=1731;
				end
			// 这个时间要足够长，延时等待让这个复位应答阶段结
			1735:begin
					DelayCnt1<=DelayCnt1+1;
					if (DelayCnt1==100)
						begin
							DelayCnt2<=DelayCnt2+1;
							if (DelayCnt2==100)
								begin
									Cnt<=1740;//跳到下一个阶段 
									DelayCnt1<=0;
									DelayCnt2<=0;
								end
							else
								begin
									Cnt<=1735;//继续延时
									DelayCnt1<=0;
								end
						end
					else
						begin
							Cnt<=1735;//继续延时
						end
				 end	
	
	
			1740:begin  DataTmp<=8'HCC;BitCnt<=0;end //忽略ROM匹配，复位后接着必是写该ROM命令
			1746:begin En<=1'b1;DQ_Reg<=1'b0;end//保持总线低电平10us
			1757:begin DQ_Reg<=DataTmp[BitCnt];end//LSB first  每个数据维持80us左右,让18b20采样(采样时间是第15～第45us)
			1860:begin DQ_Reg<=1'b1;end //拉高
			1861:begin BitCnt<=BitCnt+1'b1;end
			1862:begin
						if (BitCnt==8)
							begin
								BitCnt=0;
								Cnt<=2105; //跳到延时等待程序
							end 
						else
							begin
								Cnt<=1746;//继续发送下一个bit
							end 
					 end	
		
		
			2105:begin  DataTmp<=8'H44;BitCnt<=0;end//温度转换		
			2111:begin En<=1'b1;DQ_Reg<=1'b0;end//保持总线低电平10us
			2122:begin DQ_Reg<=DataTmp[BitCnt];end//LSB first  每个数据维持80us左右,让18b20采样(采样时间是第15～第45us)
			2220:begin DQ_Reg<=1'b1;end //拉高
			2221:begin BitCnt<=BitCnt+1'b1;end
			2225:begin
					if (BitCnt==8)
						begin
							BitCnt=0;
							Cnt<=2230; //跳到延时等待程序
						end 
					else
						begin
							Cnt<=2111;//继续发送下一个bit
						end 
				 end	
			
			//测量等待
			2230:begin
					DelayCnt1<=DelayCnt1+1;
					if (DelayCnt1==1000)
						begin
							DelayCnt2<=DelayCnt2+1;
							if (DelayCnt2==1000)
								begin
									Cnt<=2235;//跳到下一个阶段  
									DelayCnt1<=0;
									DelayCnt2<=0;
								end
							else
								begin
									Cnt<=2230;//继续延时
									DelayCnt1<=0;
								end
						end
					else
						begin
							Cnt<=2230;//继续延时
						end
				 end
			

			//第三次复位
			2240:begin En<=1'b1;DQ_Reg<=1'b0;end//输出 保持总线低电平500us(至少480us)
			2740:begin En<=1'b0;end//释放总线120us,自动拉高(等待15~60us之间,从端拉低维持60~240us之间)
			2861:begin
					if (DQ==1'b0) //为低，初始完毕 跳出
						begin
							Cnt<=2863;
							//Led[2]<=1'b1;
						end
					else		//继续等待 检测
						Cnt<=2861;
				end
	
			// 这个时间要足够长，延时等待让这个复位应答阶段结
			2863:begin
						DelayCnt1<=DelayCnt1+1;
						if (DelayCnt1==1000)
							begin
								DelayCnt2<=DelayCnt2+1;
								if (DelayCnt2==100)
									begin
										Cnt<=2865;//跳到下一个阶段 
										DelayCnt1<=0;
										DelayCnt2<=0;
									end
								else
									begin
										Cnt<=2863;//继续延时
										DelayCnt1<=0;
									end
							end
						else
							begin
								Cnt<=2863;//继续延时
							end
					 end	
	
		
			2865:begin DataTmp<=8'HCC;BitCnt<=0;end////忽略ROM匹配，复位后接着必是写该ROM命令	
			2876:begin En<=1'b1;DQ_Reg<=1'b0;end//保持总线低电平10us
			2877:begin DQ_Reg<=DataTmp[BitCnt];end//LSB first  每个数据维持80us左右,让18b20采样(采样时间是第15～第45us)
			2980:begin DQ_Reg<=1'b1;end //拉高
			2981:begin BitCnt<=BitCnt+1'b1;end
			2982:begin
					if (BitCnt==8)
						begin
							BitCnt=0;
							Cnt<=3000; //跳到发送读取温度指令
							//Led[0]<=1'b1;
						end 
					else
						begin
							Cnt<=2876;//继续发送下一个bit
						end 
				 end		
		
	

	
			//发送读取温度指令
			3000:begin  DataTmp<=8'HBE;BitCnt<=0;end//读取reg	
			3011:begin En<=1'b1;DQ_Reg<=1'b0;end//保持总线低电平10us
			3022:begin DQ_Reg<=DataTmp[BitCnt];end//LSB first  每个数据维持80us左右,让18b20采样(采样时间是第15～第45us)
			3120:begin DQ_Reg<=1'b1;end //拉高
			3121:begin BitCnt<=BitCnt+1'b1;end
			3122:begin
					if (BitCnt==8)
						begin
							BitCnt=0;
							Cnt<=3130; //跳到读取温度
							//Led[0]<=1'b1;
						end 
					else
						begin
							Cnt<=3011;//继续发送下一个bit
						end 
				 end	
			
			//读取温度  低字节
			3131:begin En<=1'b1;DQ_Reg<=1'b0;end//保持总线低电平5us(拉低至少1us)
			3135:begin En<=1'b0;end//输入  准备读取
			3144:begin ReadData[BitCnt]<=DQ;end //读取数据 LSB first	在第14us开始采样数据,并延时大约40us完成读操作
			3191:begin En<=1'b1;DQ_Reg<=1'b1;end//延时完毕，拉高DQ，准备下一次采样数据	
			3200:begin BitCnt<=BitCnt+1'b1;end	
			3201:begin
					if (BitCnt==8)
						begin
							BitCnt=0;
							Cnt<=3221; //
							TL<=ReadData;
							//Led[1]<=1'b1;
						end 
					else
						begin
							Cnt<=3131;//继续接收下一个bit
						end 	
				end	 
			//读取温度  高字节
			3221:begin En<=1'b1;DQ_Reg<=1'b0;end//保持总线低电平5us(拉低至少1us)
			3226:begin En<=1'b0;end//输入  准备读取
			3236:begin ReadData[BitCnt]<=DQ;end //读取数据 LSB first	在第14us开始采样数据,并延时大约40us完成读操作
			3285:begin En<=1'b1;DQ_Reg<=1'b1;end//输出 延时完毕，拉高DQ，准备下一次采样数据	
			3280:begin BitCnt<=BitCnt+1'b1;end	
			3281:begin
					if (BitCnt==8)
						begin
							BitCnt=0;
							Cnt<=3295; //跳到延时等待程序
							TH<=ReadData;
						end 
					else
						begin
							Cnt<=3221;//继续接收下一个bit
						end 	
				end	 
			
			//发送
			3300:begin TXData<=TH;end //发送高字节  th
			3301:begin XmitPlus<=1'b1;end	
			3342:begin XmitPlus<=1'b0;end
			3343:begin
					if (TxFinish==1'b1)
						begin
							Cnt<=4500;//发送下一个字节
							
						end
					else
						begin
							Cnt<=3343;//继续等待
						end
					end
			//要等串口发送回到IDLE状态后再发送数据 所以上一个数据发送完毕后 延时一段时间再发送下个字节
			7500:begin TXData<=TL;end //发送低字节 tl
			7501:begin XmitPlus<=1'b1;end	
			7542:begin XmitPlus<=1'b0;end
			7543:begin
					if (TxFinish==1'b1)
						begin
						Cnt<=7600;//跳到下一个阶段
						Led[2]<=1'b1;
						end
					else
						Cnt<=7543;//继续等待
				end	
			
			//////////输出温度值/////////////////////
			7600:begin Data16b<={TH,TL};end
			7610:begin FinishFlag<=1'b1;end
			7620:begin FinishFlag<=1'b0;end
			///////////////////////////////
		
			7630:begin Cnt<=1130;end//下一次温度采样开始 
			default:;	
		endcase
	end
end
	
endmodule