 //实现的动态数码管显示
module SMDisplay(Clk,Rst,Data16bIn,SMData,SMCom);//SM1,SM2,SM3,SM4,Data10b

input Clk,Rst;
input[15:0] Data16bIn;
output reg [7:0] SMData;//数码管段码显示
output reg [3:0] SMCom;//数码管位码显示

wire [9:0] Data10b;//截取的10b数据
reg [15:0] CntScan;//动态扫描的速度控制
wire [7:0] SM1Code,SM2Code,SM3Code,SM4Code;//各位数码管显示的段码
reg [3:0] SM1,SM2,SM3,SM4;//待转换的数码管十进制值


//数码管1 段码转换  SM1-小数百分位
SMData2Display Gen_SM1Code(.Rst(Rst),.Clk(Clk),.DataIn(SM1),.DataOut(SM1Code),.DpSwitch(1'b0));//小数点灭
//数码管2 段码转换  SM2-小数十分位
SMData2Display Gen_SM2Code(.Rst(Rst),.Clk(Clk),.DataIn(SM2),.DataOut(SM2Code),.DpSwitch(1'b0));//小数点灭
//数码管3 段码转换  SM3-整数个位
SMData2Display Gen_SM3Code(.Rst(Rst),.Clk(Clk),.DataIn(SM3),.DataOut(SM3Code),.DpSwitch(1'b1));//小数点亮
//数码管4 段码转换  SM4-整数十位
SMData2Display Gen_SM4Code(.Rst(Rst),.Clk(Clk),.DataIn(SM4),.DataOut(SM4Code),.DpSwitch(1'b0));//小数点灭

always@(posedge Clk or negedge  Rst)
begin
    if(!Rst) 
		begin //低电平复位
        	CntScan<=0;
     	end
    else 
		begin
        	CntScan<=CntScan+1;
        end
end

always @(CntScan)//段码扫描频率
begin 
  case(CntScan[15:14])
      2'b00 :
          SMCom = 4'b1110;
      2'b01 :
          SMCom = 4'b1101;
      2'b10 :
          SMCom = 4'b1011;
      2'b11 :
          SMCom = 4'b0111;
      default :;
    endcase
end

always@(SMCom) //对应COM信号给出各段数据，段码
begin
    case(SMCom)
        4'b1110:
            SMData=SM1Code;//输入将要显示的数字
        4'b1101:
            SMData=SM2Code;
        4'b1011:
            SMData=SM3Code;
        4'b0111:
            SMData=SM4Code;   
        default:
            SMData=8'HFF;//全灭
     endcase
end

assign Data10b=Data16bIn[9:0];//只取低10位数据

//将10位二进制数分离为SM1,SN2，SM3.SM4
always @ (posedge Clk or negedge Rst)
begin
	if (Rst==1'b0) 
		begin
			SM1<=15;
			SM2<=15;
			SM3<=15;
			SM4<=15;
		end
	else
		begin
			//整数部分
			if (Data10b[9:4]>=0 &&  Data10b[9:4]<=9)//0-9
				begin
					SM4<=0;
					SM3<=Data10b[9:4];
				end
			else if (Data10b[9:4]>=10 &&  Data10b[9:4]<=19)//10-19
				begin
					SM4<=1;
					SM3<=Data10b[9:4]-10;
				end
			else if (Data10b[9:4]>=20 &&  Data10b[9:4]<=29)//20-29
				begin
					SM4<=2;
					SM3<=Data10b[9:4]-20;
				end
			else if (Data10b[9:4]>=30 &&  Data10b[9:4]<=39)//30-39
				begin
					SM4<=3;
					SM3<=Data10b[9:4]-30;
				end
			else if (Data10b[9:4]>=40 &&  Data10b[9:4]<=49)//40-49
				begin
					SM4<=4;
					SM3<=Data10b[9:4]-40;
				end
			else if (Data10b[9:4]>=50 &&  Data10b[9:4]<=59)//50-59
				begin
					SM4<=5;
					SM3<=Data10b[9:4]-50;
				end
			else if (Data10b[9:4]>=60 &&  Data10b[9:4]<=69)//60-69
				begin
					SM4<=6;
					SM3<=Data10b[9:4]-60;
				end
			//小数部分
			case (Data10b[3:0])
				0:begin SM2<=0;SM1<=0;end//--0.00
				1:begin SM2<=0;SM1<=6;end//--0.0625
				2:begin SM2<=1;SM1<=2;end//--0.125
				3:begin SM2<=1;SM1<=8;end//--0.1875
				4:begin SM2<=2;SM1<=5;end//--0.25
				5:begin SM2<=3;SM1<=1;end//--0.3125
				6:begin SM2<=3;SM1<=7;end//--0.375
				7:begin SM2<=4;SM1<=3;end//--0.4375
				8:begin SM2<=0;SM1<=5;end//--0.5;
				9:begin SM2<=5;SM1<=6;end//--0.5625;
				10:begin SM2<=6;SM1<=2;end//--0.625:
				11:begin SM2<=6;SM1<=8;end//--0.6875:
				12:begin SM2<=7;SM1<=5;end//--0.75;:
				13:begin SM2<=8;SM1<=1;end// --0.8125;:
				14:begin SM2<=8;SM1<=7;end//--0.875;:
				15:begin SM2<=9;SM1<=3;end//--0.9375:
				default:;
			endcase			
		end

end 
endmodule


